"""Pylons middleware initialization"""
from paste.cascade import Cascade
from paste.registry import RegistryManager
from paste.urlparser import StaticURLParser
from paste.deploy.converters import asbool

from pylons import config
from pylons.error import error_template
from pylons.middleware import error_mapper, ErrorDocuments, ErrorHandler, \
    StaticJavascripts
from pylons.wsgiapp import PylonsApp, PylonsBaseWSGIApp
from django.core.handlers.wsgi import WSGIHandler

from quickwiki.config.environment import load_environment

class DjangoWrapper(PylonsBaseWSGIApp):
    def __init__(self, *args, **kwargs):
        super(DjangoWrapper, self).__init__(*args, **kwargs)
        self.django_app = WSGIHandler()
        
    def __call__(self, environ, start_response):
        try:
            environ['wsgi.input'].seek(0)
        except AttributeError:
            pass 
        self.setup_app_env(environ, start_response)
        return self.django_app(environ, start_response)

def make_app(global_conf, full_stack=True, **app_conf):
    """Create a Pylons WSGI application and return it

    ``global_conf``
        The inherited configuration for this application. Normally from
        the [DEFAULT] section of the Paste ini file.

    ``full_stack``
        Whether or not this application provides a full WSGI stack (by
        default, meaning it handles its own exceptions and errors).
        Disable full_stack when this application is "managed" by
        another WSGI middleware.

    ``app_conf``
        The application's local configuration. Normally specified in the
        [app:<name>] section of the Paste ini file (where <name>
        defaults to main).
    """
    # Configure the Pylons environment
    load_environment(global_conf, app_conf)

    # The Pylons WSGI app
    app = PylonsApp()

    # CUSTOM MIDDLEWARE HERE (filtered by error handling middlewares)

    if asbool(full_stack):
        # Handle Python exceptions
        app = ErrorHandler(app, global_conf, error_template=error_template,
                           **config['pylons.errorware'])

        # Display error documents for 401, 403, 404 status codes (and
        # 500 when debug is disabled)
        app = ErrorDocuments(app, global_conf, mapper=error_mapper, **app_conf)

    # Establish the Registry for this application
    app = RegistryManager(app)

    # Static files
    javascripts_app = StaticJavascripts()
    static_app = StaticURLParser(config['pylons.paths']['static_files'])
    forum = PylonsApp(base_wsgi_app=DjangoWrapper)
    forum = RegistryManager(forum)
    app = Cascade([static_app, javascripts_app, forum, app])
    return app
